package com.raywenderlich.android.cheesefinder.database;

import android.database.Cursor;
import androidx.room.EntityDeletionOrUpdateAdapter;
import androidx.room.EntityInsertionAdapter;
import androidx.room.RoomDatabase;
import androidx.room.RoomSQLiteQuery;
import androidx.room.util.CursorUtil;
import androidx.room.util.DBUtil;
import androidx.sqlite.db.SupportSQLiteStatement;
import java.lang.Long;
import java.lang.Override;
import java.lang.String;
import java.lang.SuppressWarnings;
import java.util.ArrayList;
import java.util.List;

@SuppressWarnings({"unchecked", "deprecation"})
public final class CheeseDao_Impl implements CheeseDao {
  private final RoomDatabase __db;

  private final EntityInsertionAdapter __insertionAdapterOfCheese;

  private final EntityDeletionOrUpdateAdapter __updateAdapterOfCheese;

  public CheeseDao_Impl(RoomDatabase __db) {
    this.__db = __db;
    this.__insertionAdapterOfCheese = new EntityInsertionAdapter<Cheese>(__db) {
      @Override
      public String createQuery() {
        return "INSERT OR IGNORE INTO `cheeses`(`id`,`name`,`favorite`) VALUES (nullif(?, 0),?,?)";
      }

      @Override
      public void bind(SupportSQLiteStatement stmt, Cheese value) {
        stmt.bindLong(1, value.getId());
        if (value.getName() == null) {
          stmt.bindNull(2);
        } else {
          stmt.bindString(2, value.getName());
        }
        stmt.bindLong(3, value.getFavorite());
      }
    };
    this.__updateAdapterOfCheese = new EntityDeletionOrUpdateAdapter<Cheese>(__db) {
      @Override
      public String createQuery() {
        return "UPDATE OR ABORT `cheeses` SET `id` = ?,`name` = ?,`favorite` = ? WHERE `id` = ?";
      }

      @Override
      public void bind(SupportSQLiteStatement stmt, Cheese value) {
        stmt.bindLong(1, value.getId());
        if (value.getName() == null) {
          stmt.bindNull(2);
        } else {
          stmt.bindString(2, value.getName());
        }
        stmt.bindLong(3, value.getFavorite());
        stmt.bindLong(4, value.getId());
      }
    };
  }

  @Override
  public List<Long> insertAll(final List<Cheese> cheeses) {
    __db.assertNotSuspendingTransaction();
    __db.beginTransaction();
    try {
      List<Long> _result = __insertionAdapterOfCheese.insertAndReturnIdsList(cheeses);
      __db.setTransactionSuccessful();
      return _result;
    } finally {
      __db.endTransaction();
    }
  }

  @Override
  public int favoriteCheese(final Cheese cheese) {
    __db.assertNotSuspendingTransaction();
    int _total = 0;
    __db.beginTransaction();
    try {
      _total +=__updateAdapterOfCheese.handle(cheese);
      __db.setTransactionSuccessful();
      return _total;
    } finally {
      __db.endTransaction();
    }
  }

  @Override
  public List<Cheese> findCheese(final String name) {
    final String _sql = "SELECT * FROM cheeses WHERE name LIKE ?";
    final RoomSQLiteQuery _statement = RoomSQLiteQuery.acquire(_sql, 1);
    int _argIndex = 1;
    if (name == null) {
      _statement.bindNull(_argIndex);
    } else {
      _statement.bindString(_argIndex, name);
    }
    __db.assertNotSuspendingTransaction();
    final Cursor _cursor = DBUtil.query(__db, _statement, false);
    try {
      final int _cursorIndexOfId = CursorUtil.getColumnIndexOrThrow(_cursor, "id");
      final int _cursorIndexOfName = CursorUtil.getColumnIndexOrThrow(_cursor, "name");
      final int _cursorIndexOfFavorite = CursorUtil.getColumnIndexOrThrow(_cursor, "favorite");
      final List<Cheese> _result = new ArrayList<Cheese>(_cursor.getCount());
      while(_cursor.moveToNext()) {
        final Cheese _item;
        final long _tmpId;
        _tmpId = _cursor.getLong(_cursorIndexOfId);
        final String _tmpName;
        _tmpName = _cursor.getString(_cursorIndexOfName);
        final int _tmpFavorite;
        _tmpFavorite = _cursor.getInt(_cursorIndexOfFavorite);
        _item = new Cheese(_tmpId,_tmpName,_tmpFavorite);
        _result.add(_item);
      }
      return _result;
    } finally {
      _cursor.close();
      _statement.release();
    }
  }

  @Override
  public int isFavorite(final long id) {
    final String _sql = "SELECT favorite FROM cheeses WHERE ? LIMIT 1";
    final RoomSQLiteQuery _statement = RoomSQLiteQuery.acquire(_sql, 1);
    int _argIndex = 1;
    _statement.bindLong(_argIndex, id);
    __db.assertNotSuspendingTransaction();
    final Cursor _cursor = DBUtil.query(__db, _statement, false);
    try {
      final int _result;
      if(_cursor.moveToFirst()) {
        _result = _cursor.getInt(0);
      } else {
        _result = 0;
      }
      return _result;
    } finally {
      _cursor.close();
      _statement.release();
    }
  }
}
