package com.raywenderlich.android.cheesefinder.database;

import androidx.room.DatabaseConfiguration;
import androidx.room.InvalidationTracker;
import androidx.room.RoomOpenHelper;
import androidx.room.RoomOpenHelper.Delegate;
import androidx.room.util.DBUtil;
import androidx.room.util.TableInfo;
import androidx.room.util.TableInfo.Column;
import androidx.room.util.TableInfo.ForeignKey;
import androidx.room.util.TableInfo.Index;
import androidx.sqlite.db.SupportSQLiteDatabase;
import androidx.sqlite.db.SupportSQLiteOpenHelper;
import androidx.sqlite.db.SupportSQLiteOpenHelper.Callback;
import androidx.sqlite.db.SupportSQLiteOpenHelper.Configuration;
import java.lang.IllegalStateException;
import java.lang.Override;
import java.lang.String;
import java.lang.SuppressWarnings;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

@SuppressWarnings({"unchecked", "deprecation"})
public final class CheeseDatabase_Impl extends CheeseDatabase {
  private volatile CheeseDao _cheeseDao;

  @Override
  protected SupportSQLiteOpenHelper createOpenHelper(DatabaseConfiguration configuration) {
    final SupportSQLiteOpenHelper.Callback _openCallback = new RoomOpenHelper(configuration, new RoomOpenHelper.Delegate(1) {
      @Override
      public void createAllTables(SupportSQLiteDatabase _db) {
        _db.execSQL("CREATE TABLE IF NOT EXISTS `cheeses` (`id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, `name` TEXT NOT NULL, `favorite` INTEGER NOT NULL)");
        _db.execSQL("CREATE UNIQUE INDEX `index_cheeses_name` ON `cheeses` (`name`)");
        _db.execSQL("CREATE TABLE IF NOT EXISTS room_master_table (id INTEGER PRIMARY KEY,identity_hash TEXT)");
        _db.execSQL("INSERT OR REPLACE INTO room_master_table (id,identity_hash) VALUES(42, \"480b8fa7790ae7f6ea77798efec2d8e3\")");
      }

      @Override
      public void dropAllTables(SupportSQLiteDatabase _db) {
        _db.execSQL("DROP TABLE IF EXISTS `cheeses`");
      }

      @Override
      protected void onCreate(SupportSQLiteDatabase _db) {
        if (mCallbacks != null) {
          for (int _i = 0, _size = mCallbacks.size(); _i < _size; _i++) {
            mCallbacks.get(_i).onCreate(_db);
          }
        }
      }

      @Override
      public void onOpen(SupportSQLiteDatabase _db) {
        mDatabase = _db;
        internalInitInvalidationTracker(_db);
        if (mCallbacks != null) {
          for (int _i = 0, _size = mCallbacks.size(); _i < _size; _i++) {
            mCallbacks.get(_i).onOpen(_db);
          }
        }
      }

      @Override
      public void onPreMigrate(SupportSQLiteDatabase _db) {
        DBUtil.dropFtsSyncTriggers(_db);
      }

      @Override
      public void onPostMigrate(SupportSQLiteDatabase _db) {
      }

      @Override
      protected void validateMigration(SupportSQLiteDatabase _db) {
        final HashMap<String, TableInfo.Column> _columnsCheeses = new HashMap<String, TableInfo.Column>(3);
        _columnsCheeses.put("id", new TableInfo.Column("id", "INTEGER", true, 1));
        _columnsCheeses.put("name", new TableInfo.Column("name", "TEXT", true, 0));
        _columnsCheeses.put("favorite", new TableInfo.Column("favorite", "INTEGER", true, 0));
        final HashSet<TableInfo.ForeignKey> _foreignKeysCheeses = new HashSet<TableInfo.ForeignKey>(0);
        final HashSet<TableInfo.Index> _indicesCheeses = new HashSet<TableInfo.Index>(1);
        _indicesCheeses.add(new TableInfo.Index("index_cheeses_name", true, Arrays.asList("name")));
        final TableInfo _infoCheeses = new TableInfo("cheeses", _columnsCheeses, _foreignKeysCheeses, _indicesCheeses);
        final TableInfo _existingCheeses = TableInfo.read(_db, "cheeses");
        if (! _infoCheeses.equals(_existingCheeses)) {
          throw new IllegalStateException("Migration didn't properly handle cheeses(com.raywenderlich.android.cheesefinder.database.Cheese).\n"
                  + " Expected:\n" + _infoCheeses + "\n"
                  + " Found:\n" + _existingCheeses);
        }
      }
    }, "480b8fa7790ae7f6ea77798efec2d8e3", "2d3739023786abe95f86426f79a820d7");
    final SupportSQLiteOpenHelper.Configuration _sqliteConfig = SupportSQLiteOpenHelper.Configuration.builder(configuration.context)
        .name(configuration.name)
        .callback(_openCallback)
        .build();
    final SupportSQLiteOpenHelper _helper = configuration.sqliteOpenHelperFactory.create(_sqliteConfig);
    return _helper;
  }

  @Override
  protected InvalidationTracker createInvalidationTracker() {
    final HashMap<String, String> _shadowTablesMap = new HashMap<String, String>(0);
    HashMap<String, Set<String>> _viewTables = new HashMap<String, Set<String>>(0);
    return new InvalidationTracker(this, _shadowTablesMap, _viewTables, "cheeses");
  }

  @Override
  public void clearAllTables() {
    super.assertNotMainThread();
    final SupportSQLiteDatabase _db = super.getOpenHelper().getWritableDatabase();
    try {
      super.beginTransaction();
      _db.execSQL("DELETE FROM `cheeses`");
      super.setTransactionSuccessful();
    } finally {
      super.endTransaction();
      _db.query("PRAGMA wal_checkpoint(FULL)").close();
      if (!_db.inTransaction()) {
        _db.execSQL("VACUUM");
      }
    }
  }

  @Override
  public CheeseDao cheeseDao() {
    if (_cheeseDao != null) {
      return _cheeseDao;
    } else {
      synchronized(this) {
        if(_cheeseDao == null) {
          _cheeseDao = new CheeseDao_Impl(this);
        }
        return _cheeseDao;
      }
    }
  }
}
